from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist
from rest_framework import serializers
from rest_framework.fields import SerializerMethodField
from rest_framework_gis.fields import GeometryField
from versatileimagefield.serializers import VersatileImageFieldSerializer

from terracommon.datastore.serializers import RelatedDocumentFileSerializer
from terracommon.terra.models import Feature, Layer

from .models import Campaign, Picture, Viewpoint

UserModel = get_user_model()


class PermissiveImageFieldSerializer(VersatileImageFieldSerializer):
    def get_attribute(self, instance):
        try:
            return super().get_attribute(instance)
        except (AttributeError, ObjectDoesNotExist):
            # Will silence any NoneType or failing query on attribute
            return None


class SimpleViewpointSerializer(serializers.ModelSerializer):
    picture = SerializerMethodField()
    geometry = GeometryField(source='point.geom', read_only=True)

    class Meta:
        model = Viewpoint
        fields = ('id', 'label', 'picture', 'geometry')

    def get_picture(self, viewpoint):
        try:
            return VersatileImageFieldSerializer('tropp').to_native(
                viewpoint.ordered_pics[0].file
            )
        except IndexError:
            return None


class SimpleAuthenticatedViewpointSerializer(SimpleViewpointSerializer):
    status = serializers.SerializerMethodField()

    class Meta:
        model = Viewpoint
        fields = ('id', 'label', 'picture', 'geometry', 'status')

    def get_status(self, obj):
        """
        :return: string (missing, draft, submitted, accepted)
        """
        # Get only pictures created for the campaign
        try:
            last_pic = obj.ordered_pics[0]
            if last_pic.created_at < obj.created_at:
                return settings.STATES.CHOICES_DICT[settings.STATES.MISSING]
            return settings.STATES.CHOICES_DICT[last_pic.state]
        except IndexError:
            return settings.STATES.CHOICES_DICT[settings.STATES.MISSING]


class CampaignSerializer(serializers.ModelSerializer):
    owner = serializers.ReadOnlyField(source='owner.email')

    class Meta:
        model = Campaign
        fields = '__all__'


class DetailCampaignNestedSerializer(serializers.ModelSerializer):
    owner = serializers.ReadOnlyField(source='owner.email')
    viewpoints = SimpleViewpointSerializer(many=True, read_only=True)

    class Meta(CampaignSerializer.Meta):
        model = Campaign
        fields = '__all__'


class DetailAuthenticatedCampaignNestedSerializer(serializers.ModelSerializer):
    owner = serializers.ReadOnlyField(source='owner.email')
    viewpoints = SimpleAuthenticatedViewpointSerializer(many=True, read_only=True)

    class Meta(CampaignSerializer.Meta):
        model = Campaign
        fields = '__all__'


class ListCampaignNestedSerializer(CampaignSerializer):
    picture = PermissiveImageFieldSerializer(
        'tropp',
        source='viewpoints.first.pictures.first.file',
    )
    # Override to expose typed data
    statistics = serializers.DictField(
        child=serializers.IntegerField(),
        read_only=True,
    )
    status = serializers.BooleanField(read_only=True)

    class Meta(CampaignSerializer.Meta):
        model = Campaign
        fields = ('label', 'assignee', 'picture', 'statistics', 'status')


class PictureSerializer(serializers.ModelSerializer):
    owner = serializers.ReadOnlyField(source='owner.email')

    class Meta:
        model = Picture
        fields = '__all__'


class SimplePictureSerializer(PictureSerializer):
    file = VersatileImageFieldSerializer('tropp')

    class Meta:
        model = Picture
        fields = ('id', 'date', 'file', 'owner', 'properties')


class ViewpointSerializerWithPicture(serializers.ModelSerializer):
    picture = SimplePictureSerializer(required=False, write_only=True)
    pictures = SimplePictureSerializer(many=True, read_only=True)
    related = RelatedDocumentFileSerializer(many=True, read_only=True)
    point = GeometryField(required=True, write_only=True)
    geometry = GeometryField(source='point.geom', read_only=True)

    class Meta:
        model = Viewpoint
        fields = ('id', 'label', 'geometry', 'properties', 'point', 'picture',
                  'pictures', 'related')

    def create(self, validated_data):
        point_data = validated_data.pop('point', None)
        layer, created = Layer.objects.get_or_create(
            name=settings.TROPP_BASE_LAYER_NAME
        )
        feature = Feature.objects.create(
            geom=point_data,
            layer=layer,
            properties={},
        )
        validated_data.setdefault('point', feature)

        picture_data = validated_data.pop('picture', None)
        viewpoint = super().create(validated_data)
        if picture_data:
            Picture.objects.create(
                viewpoint=viewpoint,
                owner=self.context['request'].user,
                **picture_data,
            )

        return viewpoint

    def update(self, instance, validated_data):
        picture_data = validated_data.pop('picture', None)
        if picture_data:
            Picture.objects.create(
                viewpoint=instance,
                owner=self.context['request'].user,
                **picture_data,
            )

        point_data = validated_data.pop('point', None)
        if point_data:
            feature = instance.point
            feature.geom = point_data
            feature.save()

        return super().update(instance, validated_data)


class ViewpointLabelSerializer(serializers.ModelSerializer):
    class Meta:
        model = Viewpoint
        fields = ('id', 'label')


class PhotographerLabelSerializer(serializers.ModelSerializer):
    label = serializers.CharField(source='__str__')

    class Meta:
        model = get_user_model()
        fields = ('uuid', 'label')



from django.contrib.auth import get_user_model
from django.db import models

from .managers import NotificationManager

UserModel = get_user_model()

LEVELS = (
    ('DEBUG', 'Debug'),
    ('INFO', 'Information'),
    ('SUCCESS', 'Succès'),
    ('WARNING', 'Avertissement'),
    ('ERROR', 'Erreur')
)


class UserNotifications(models.Model):
    user = models.ForeignKey(UserModel,
                             on_delete=models.PROTECT,
                             related_name='notifications')
    created_at = models.DateTimeField(auto_now_add=True)
    read = models.BooleanField(default=False)
    level = models.CharField(choices=LEVELS, max_length=255, blank=False)
    event_code = models.CharField(max_length=255)
    message = models.TextField(blank=True)
    identifier = models.IntegerField()
    uuid = models.UUIDField(null=True)

    objects = NotificationManager()

    class Meta:
        ordering = ['id']



# Generated by Django 2.0.9 on 2018-11-20 10:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0003_auto_20181023_1522'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='readmodel',
            options={'ordering': ['id']},
        ),
        migrations.AlterModelOptions(
            name='terrauser',
            options={'ordering': ['id'], 'permissions': (('can_manage_users', 'Is able create, delete, update users'),)},
        ),
    ]



from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.contrib.postgres.fields import JSONField
from django.db import models
from django.db.models.manager import BaseManager

from .managers import DataStoreQuerySet


class DataStore(models.Model):
    key = models.CharField(max_length=255, primary_key=True, )
    value = JSONField(default=dict, blank=False)

    objects = BaseManager.from_queryset(DataStoreQuerySet)()

    class Meta:
        ordering = ['key']


class DataStorePermission(models.Model):
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    prefix = models.CharField(max_length=255, blank=False, null=False)
    permission = models.ForeignKey(Permission, on_delete=models.CASCADE)

    class Meta:
        ordering = ['prefix']
        permissions = (
            ('can_read_datastore', "Is able to read all datastore's elements"),
            ('can_readwrite_datastore', 'Is able to write in datastore'),
        )


def related_document_path(instance, filename):
    return (f'documents/'
            f'{instance.content_type.app_label}_{instance.content_type.model}'
            f'/{instance.object_id}/{instance.key}')


class RelatedDocument(models.Model):
    key = models.CharField(max_length=255, blank=False)
    content_type = models.ForeignKey(ContentType, on_delete=models.PROTECT)
    object_id = models.PositiveIntegerField()
    linked_object = GenericForeignKey('content_type', 'object_id')
    document = models.FileField(upload_to=related_document_path, null=False)
    properties = JSONField(default=dict)

    class Meta:
        ordering = ['key']
        unique_together = ('key', 'content_type', 'object_id')



# Generated by Django 2.0.8 on 2018-08-17 08:01

import django.contrib.postgres.fields.jsonb
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import terracommon.accounts.mixins
import terracommon.trrequests.helpers


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('terra', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('properties', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict)),
                ('is_internal', models.BooleanField(default=False)),
                ('attachment', models.FileField(blank=True, upload_to=terracommon.trrequests.helpers.rename_comment_attachment)),
                ('filename', models.CharField(blank=True, editable=False, help_text='Initial name of the attachment', max_length=255)),
                ('layer', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='terra.Layer')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='UserRequest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('expiry', models.DateField(default=None, null=True)),
                ('state', models.IntegerField(default=100)),
                ('properties', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict)),
                ('layer', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='userrequests', to='terra.Layer')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='userrequests', to=settings.AUTH_USER_MODEL)),
                ('reviewers', models.ManyToManyField(blank=True, related_name='to_review', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'permissions': (('can_create_requests', 'Is able to create a new requests'), ('can_read_self_requests', 'Is able to get own requests'), ('can_read_all_requests', 'Is able to get all requests'), ('can_comment_requests', 'Is able to comment an user request'), ('can_internal_comment_requests', 'Is able to add comments not visible by users'), ('can_read_comment_requests', 'Is allowed to read only non-internal comments'), ('can_change_state_requests', 'Is authorized to change the request state')),
            },
            bases=(models.Model, terracommon.accounts.mixins.ReadableModelMixin),
        ),
        migrations.AddField(
            model_name='comment',
            name='userrequest',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='comments', to='trrequests.UserRequest'),
        ),
    ]



import logging

from django.apps import apps
from django.core.management.base import BaseCommand
from django.db import transaction
from django.utils.functional import cached_property
from django.utils.module_loading import import_string

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Populate database from all installed app's initialization modules"

    POPULATE_MODULE_NAME = 'populate'
    POPULATE_FN_NAME = 'load_data'
    POPULATE_TEST_FN_NAME = 'load_test_data'

    def add_arguments(self, parser):
        parser.add_argument('-d', '--dry-run',
                            action="store_true",
                            help='Dry-run mode')
        parser.add_argument('-t', '--test-data',
                            action="store_true",
                            help='Load test data')
        parser.add_argument('-l', '--list',
                            action="store_true",
                            help='List available modules')
        parser.add_argument('-m', '--modules',
                            action="store",
                            nargs="?",
                            help="Load data for this modules")

    def handle(self, *args, **options):
        self.test_mode = True if options.get('test_data') else False

        if options.get('list', False):
            self.stdout.write('Applications with populate modules:')
            for app_name in self.available_modules.keys():
                self.stdout.write('  - {}'.format(app_name))
            exit(0)

        sid = transaction.savepoint()

        for app_name, load_data_fn in self.available_modules.items():
            with transaction.atomic():
                self.stdout.write('Loading data for {}'.format(app_name))
                load_data_fn()

        if options.get('dry_run'):
            transaction.savepoint_rollback(sid)

    @cached_property
    def available_modules(self):
        available_modules = {}

        fn_name = self.POPULATE_TEST_FN_NAME \
            if self.test_mode else self.POPULATE_FN_NAME

        for app_name, app_config in apps.app_configs.items():
            try:
                available_modules[app_name] = import_string("{}.{}.{}".format(
                    app_config.module.__package__,
                    self.POPULATE_MODULE_NAME,
                    fn_name))
            except ImportError:
                logger.debug('Application {} has no'
                             ' populate module'.format(app_name))

        return available_modules



# Generated by Django 2.0.8 on 2018-08-17 08:01

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserNotifications',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('read', models.BooleanField(default=False)),
                ('level', models.CharField(choices=[('DEBUG', 'Debug'), ('INFO', 'Information'), ('SUCCESS', 'Succès'), ('WARNING', 'Avertissement'), ('ERROR', 'Erreur')], max_length=255)),
                ('event_code', models.CharField(max_length=255)),
                ('message', models.TextField(blank=True)),
                ('identifier', models.IntegerField()),
                ('uuid', models.UUIDField(null=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='notifications', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 2.1.5 on 2019-01-22 14:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tropp', '0002_auto_20181123_1028'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='picture',
            options={'get_latest_by': 'date', 'ordering': ['-date'], 'permissions': (('change_state_picture', 'Is able to change the picture state'),)},
        ),
    ]



import datetime
import os


def rename_comment_attachment(instance, filename):
    ur = instance.userrequest
    ur_year = ur.created_at.year
    ur_month = ur.created_at.month
    return os.path.join(f'userrequests/{ur_year}/{ur_month:02d}/{ur.pk}',
                        f'comment_{datetime.date.today():%d}')



import json
from urllib.parse import unquote, urljoin

from django.conf import settings
from django.core.cache import cache
from django.http import HttpResponse, HttpResponseNotFound
from django.urls import reverse
from drf_yasg.utils import swagger_auto_schema
from rest_framework.views import APIView

from ..models import Feature, Layer
from .helpers import VectorTile


class TilejsonView(APIView):

    permission_classes = ()

    def _join_group_settings_link(self, layers, *args):
        return ','.join(set([
            a if 'link' not in a else
            '<a href="{0}"/>{1}</a>'.format(a['link'].replace('"', '&quot;'), a['name'].replace('"', '&quot;'))
            for a in [layer.layer_settings_with_default(*args) for layer in self.layers]
            if a
        ])) or None

    def _join_group_settings_string(self, layers, *args):
        return ','.join(set([
            a
            for a in [layer.layer_settings_with_default(*args) for layer in self.layers]
            if a
        ])) or None

    def get_tilejson(self, group):
        minzoom = max(
            settings.MIN_TILE_ZOOM,
            min(map(
                lambda l: l.layer_settings_with_default('tiles', 'minzoom'),
                self.layers)))
        maxzoom = min(
            settings.MAX_TILE_ZOOM,
            max(map(
                lambda l: l.layer_settings_with_default('tiles', 'maxzoom'),
                self.layers)))
        tile_path = reverse("terra:group-tiles-pattern", args=[group])

        # https://github.com/mapbox/tilejson-spec/tree/3.0/3.0.0
        return {
            'tilejson': '3.0.0',
            'name': group,
            'tiles': [
                unquote(urljoin(hostname, tile_path))
                for hostname in settings.TERRA_TILES_HOSTNAMES
            ],
            'minzoom': minzoom,
            'maxzoom': maxzoom,
            # bounds
            # center
            "attribution": self._join_group_settings_link(self.layers, 'metadata', 'attribution'),
            "description": self._join_group_settings_string(self.layers, 'metadata', 'description'),
            'vector_layers': [{
                'id': layer.name,
                'fields': self.layer_fields(layer),
                'minzoom': layer.layer_settings_with_default(
                    'tiles', 'minzoom'),
                'maxzoom': layer.layer_settings_with_default(
                    'tiles', 'maxzoom'),
            } for layer in self.layers]
        }

    def layer_fields(self, layer):
        properties_filter = layer.layer_settings_with_default(
            'tiles', 'properties_filter')
        if properties_filter is not None:
            fields = properties_filter
        else:
            fields = layer.layer_properties.keys()

        return {f: '' for f in fields}

    @swagger_auto_schema(
        responses={
            200: 'Returns a protobuf mapbox vector tile',
            404: 'The layer group does not exist'
            }
    )
    def get(self, request, group):
        self.layers = Layer.objects.filter(group=group)

        if self.layers.count() == 0:
            return HttpResponseNotFound()

        last_update = Feature.objects.filter(layer__in=self.layers).order_by('-updated_at').first()

        if last_update:
            cache_key = f'tilejson-{group}'
            version = int(last_update.updated_at.timestamp())
            tilejson_data = cache.get(cache_key, version=version)

            if tilejson_data is None:
                tilejson_data = json.dumps(self.get_tilejson(group))
                cache.set(cache_key, tilejson_data, version=version)

            return HttpResponse(
                tilejson_data,
                content_type='application/json')

        return HttpResponseNotFound()


class MVTView(APIView):

    permission_classes = ()

    def get_tile(self):
        big_tile = b''

        for layer in self.layers:
            minzoom = layer.layer_settings_with_default('tiles', 'minzoom')
            maxzoom = layer.layer_settings_with_default('tiles', 'maxzoom')
            if self.z >= minzoom and self.z <= maxzoom:
                feature_count, tile = self.get_tile_for_layer(layer)
                if feature_count:
                    big_tile += tile
        return big_tile

    def get_tile_for_layer(self, layer):
        tile = VectorTile(layer)
        features = layer.features.all()
        return tile.get_tile(
            self.x, self.y, self.z,
            layer.layer_settings_with_default('tiles', 'pixel_buffer'),
            layer.layer_settings_with_default('tiles', 'features_filter'),
            layer.layer_settings_with_default('tiles', 'properties_filter'),
            layer.layer_settings_with_default('tiles', 'features_limit'),
            features)

    @swagger_auto_schema(
        responses={
            200: 'Returns a protobuf mapbox vector tile',
            404: 'The layer group does not exist'
            }
    )
    def get(self, request, group, z, x, y):
        if z > settings.MAX_TILE_ZOOM or z < settings.MIN_TILE_ZOOM:
            return HttpResponse(status=204)
        self.z = z
        self.x = x
        self.y = y

        self.layers = Layer.objects.filter(group=group)

        if self.layers.count() == 0:
            return HttpResponseNotFound()

        return HttpResponse(
                    self.get_tile(),
                    content_type="application/vnd.mapbox-vector-tile"
                    )



import datetime
import hashlib
import io
import logging
import os
import shutil
import subprocess
import zipfile
from tempfile import NamedTemporaryFile, TemporaryDirectory

import jinja2
from django.conf import settings
from django.core.files import File
from django.template import Context, Template
from django.template.exceptions import \
    TemplateSyntaxError as DjangoTemplateSyntaxError
from django.utils.functional import cached_property
from docx.shared import Mm
from docxtpl import DocxTemplate, InlineImage
from jinja2 import TemplateSyntaxError
from magic import from_file
from weasyprint import HTML

from terracommon.document_generator.models import DownloadableDocument

from .filters import (b64_to_inlineimage, timedelta_filter, todate_filter,
                      translate_filter)

logger = logging.getLogger(__name__)


class DocumentGenerator:

    def __init__(self, downloadabledoc):
        if not isinstance(downloadabledoc, DownloadableDocument):
            raise TypeError("downloadabledoc must be a DownloadableDocument")
        self.template = downloadabledoc.document.documenttemplate.path
        try:
            self.type = from_file(self.template, mime=True)
        except FileNotFoundError:
            logger.warning(f"File {self.template} not found.")
            raise
        self.datamodel = downloadabledoc.linked_object
        self.mime_type_mapping = {
            'text/html': self._get_html_as_pdf,
            'application/octet-stream': self._get_docx_as_pdf,
        }

    def get_docx(self, data):
        doc = DocxTemplator(self.template)

        updated_data = (self._get_image(data, doc)
                        if data.get('documents')
                        else data)
        updated_data['tpl'] = doc  # doc is needed in a custom jinja filter
        jinja_env = jinja2.Environment()
        jinja_env.globals['now'] = datetime.datetime.now
        jinja_env.filters.update(self.filters)

        # render is perform in a temp dir
        # Because some custom jinja filter used temp files
        # which are not removed during render
        with TemporaryDirectory() as tmpdir:
            updated_data['tmpdir'] = tmpdir  # used by tempfile in custom filter
            doc.render(context=updated_data, jinja_env=jinja_env)

        # tmpdir is not removed if not empty. Ensure it's cleaned anyway
        if os.path.isdir(tmpdir):
            shutil.rmtree(tmpdir)
        return doc.save()

    def get_html(self, data):
        with open(self.template, 'r') as read_file:
            template = Template(read_file.read())
        html_content = template.render(Context(data))
        return html_content

    def get_pdf(self, reset_cache=False):
        cachepath = os.path.join(
            self.datamodel.__class__.__name__,
            f'{self._document_checksum}_{self.datamodel.pk}.pdf'
        )

        cache = CachedDocument(cachepath)
        if cache.exist:
            reset_cache = self.datamodel.updated_at.timestamp() > os.path.getmtime(cache.name)

        if not cache.exist or reset_cache:
            if reset_cache:
                cache.clear()

            serializer = (self.datamodel.get_pdf_serializer()
                          if hasattr(self.datamodel, 'get_pdf_serializer')
                          else self.datamodel.get_serializer())
            serialized_model = serializer(self.datamodel)

            self.mime_type_mapping[self.type](cache, serialized_model)

        return cache.name

    def _get_html_as_pdf(self, cache, serialized_model):
        try:
            html_content = self.get_html(serialized_model.data)
        except DjangoTemplateSyntaxError:
            cache.remove()
            logger.warning(f'TemplateSyntaxError for {self.template}')
            raise
        pdf = HTML(string=html_content).write_pdf()
        with cache.open() as cached_pdf:
            cached_pdf.write(pdf)

    def _get_docx_as_pdf(self, cache, serialized_model):
        try:
            docx = self.get_docx(data=serialized_model.data)
        except TemplateSyntaxError as e:
            cache.remove()
            logger.warning(f'TemplateSyntaxError for {self.template} '
                           f'at line {e.lineno}: {e.message}')
            raise

        # Create a temporary docx file on disk so libreoffice can use it
        with TemporaryDirectory() as tmpdir, NamedTemporaryFile(mode='wb',
                                                                dir=tmpdir,
                                                                suffix='.docx') as tmp_docx:
            tmp_docx.write(docx.getvalue())  # docx is an io.BytesIO

            # Call libreoffice to convert docx to pdf
            subprocess.run([
                'lowriter',
                '--headless',
                '--convert-to',
                'pdf:writer_pdf_Export',
                '--outdir',
                tmpdir,
                tmp_docx.name
            ])

            # Get pdf name of the file created from libreoffice writer
            tmp_pdf_root = os.path.splitext(os.path.basename(tmp_docx.name))[0]
            tmp_pdf = os.path.join(tmpdir, f'{tmp_pdf_root}.pdf')

            with cache.open() as cached_pdf, open(tmp_pdf, 'rb') as pdf:
                cached_pdf.write(pdf.read())

            # We don't need it anymore
            os.remove(tmp_pdf)

    def _get_image(self, data, tpl):
        for document in data['documents']:
            img_path = os.path.join(settings.MEDIA_ROOT, document['document'])
            # Set as image of 170mm width
            document['document'] = InlineImage(tpl, img_path, width=Mm(170))
        return data

    @cached_property
    def _document_checksum(self):
        """ return the md5 checksum of self.template """
        content = None
        if isinstance(self.template, io.IOBase):
            content = self.template.read()
        else:
            content = bytes(self.template, 'utf-8')

        return hashlib.md5(content).hexdigest()

    # TODO make it a function in filters.py
    filters = {
        'timedelta_filter': timedelta_filter,
        'translate_filter': translate_filter,
        'todate_filter': todate_filter,
        'b64_to_inlineimage': b64_to_inlineimage,
    }


class CachedDocument(File):
    cache_root = 'cache'

    def __init__(self, filename):
        self.pathname = os.path.join(self.cache_root, filename)

        if not os.path.isfile(self.pathname):
            self.exist = False

            # dirname is not current dir
            if os.path.dirname(self.pathname) != '':
                os.makedirs(os.path.dirname(self.pathname), exist_ok=True)

            super().__init__(open(self.pathname, mode='xb+'))
        else:
            self.exist = True
            super().__init__(open(self.pathname, mode='rb'))

    def remove(self):
        self.close()
        os.remove(self.name)

    def clear(self, mode=None):
        self.remove()
        self.file = open(self.name, 'xb+')


class DocxTemplator(DocxTemplate):
    def post_processing(self, docx_bytesio):
        if self.crc_to_new_media or self.crc_to_new_embedded:
            backup_bytesio = io.BytesIO()

            with zipfile.ZipFile(docx_bytesio) as zin:
                with zipfile.ZipFile(backup_bytesio, 'w') as zout:
                    for item in zin.infolist():
                        buf = zin.read(item.filename)

                        if (item.filename.startwith('word/media/')
                                and item.CRC in self.crc_to_new_media):
                            zout.writestr(item,
                                          self.crc_to_new_media[item.CRC])
                        elif (item.filename.startwith('word/embeddings/')
                              and item.CRC in self.crc_to_new_embedded):
                            zout.writestr(item,
                                          self.crc_to_new_embedded[item.CRC])
                        else:
                            zout.writestr(item, buf)
            return backup_bytesio
        return docx_bytesio

    def save(self, *args, **kwargs):
        docx_bytesio = io.BytesIO()
        self.pre_processing()
        self.docx.save(docx_bytesio)
        return self.post_processing(docx_bytesio)



# Generated by Django 2.0.9 on 2018-10-12 07:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document_generator', '0003_auto_20180911_0827'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='documenttemplate',
            options={'permissions': (('can_upload_template', 'Is allowed to upload a template'), ('can_update_template', 'Is allowed to update a template'), ('can_delete_template', 'Is allowed to delete a template'))},
        ),
        migrations.AlterField(
            model_name='documenttemplate',
            name='documenttemplate',
            field=models.FileField(upload_to='templates/%Y/%m/'),
        ),
    ]



from django.contrib.auth import get_user_model
from django.contrib.auth.forms import SetPasswordForm
from django.contrib.auth.tokens import default_token_generator
from django.contrib.auth.views import PasswordResetConfirmView
from rest_framework import serializers
from rest_framework.serializers import ValidationError

UserModel = get_user_model()


class PasswordChangeSerializer(serializers.Serializer):
    old_password = serializers.CharField(max_length=128, required=False)
    new_password1 = serializers.CharField(max_length=128)
    new_password2 = serializers.CharField(max_length=128)
    properties = serializers.JSONField(required=False)

    set_password_form_class = SetPasswordForm

    def __init__(self, user, **kwargs):
        self.user = user
        super().__init__(**kwargs)

    def validate_old_password(self, value):
        if (self.user.has_usable_password()
                and not self.user.check_password(value)):
            raise ValidationError('Invalid password')
        return value

    def validate(self, attrs):
        if not self.user:
            raise ValidationError({'User': ['Invalid user']})

        self.set_password_form = self.set_password_form_class(
            user=self.user, data=attrs
        )

        if not self.set_password_form.is_valid():
            raise ValidationError(self.set_password_form.errors)
        return attrs

    def save(self):
        user = self.set_password_form.save()
        if 'properties' in self.validated_data:
            user.properties = self.validated_data.get('properties')
            user.save()
        return user


class PasswordResetSerializer(PasswordChangeSerializer):
    check_token = default_token_generator.check_token
    get_user = PasswordResetConfirmView.get_user

    def __init__(self, uidb64, token, **kwargs):
        self.uidb64, self.token = uidb64, token
        self.user = self.get_user(self.uidb64)
        super().__init__(self.user, **kwargs)

    def validate(self, attrs):
        if not self.check_token(self.user, self.token):
            raise ValidationError({'token': ['Invalid value']})

        return super().validate(attrs)


class UserProfileSerializer(serializers.ModelSerializer):

    class Meta:
        model = UserModel
        fields = ('uuid', 'email', 'properties')
        read_only_fields = ('uuid', UserModel.USERNAME_FIELD, )


class TerraUserSerializer(serializers.ModelSerializer):
    permissions = serializers.SerializerMethodField()
    groups = serializers.SerializerMethodField()
    password = serializers.CharField(write_only=True, required=False)

    def get_groups(self, obj):
        return [group.name for group in obj.groups.all()]

    def get_permissions(self, obj):
        return list(obj.get_all_permissions())

    def save(self):
        super().save()

        if 'password' in self.validated_data:
            self.instance.set_password(self.validated_data['password'])
            self.instance.save()

        return self.instance

    class Meta:
        model = UserModel
        fields = ('id', 'is_superuser', 'email', 'uuid', 'properties',
                  'is_staff', 'is_active', 'permissions', 'groups', 'password')



# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import copy
import os
from datetime import timedelta

from django.utils.log import DEFAULT_LOGGING

from terracommon.terra.helpers import Choices

PROJECT_DIR = os.path.abspath('.')
PUBLIC_DIR = os.path.join(PROJECT_DIR, 'public')
PROJECT_NAME = os.environ.get('PROJECT_PACKAGE')
DEFAULT_FILE_STORAGE = 'django.core.files.storage.FileSystemStorage'

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.0/howto/deployment/checklist/

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

ALLOWED_HOSTS = []

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.gis',
    'rest_framework',
    'rest_framework_gis',
    'drf_yasg',
    'corsheaders',
    'storages',
)

MIDDLEWARE = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
)

ROOT_URLCONF = PROJECT_NAME + '.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(PROJECT_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.i18n',
            ],
        },
    },
]

REST_FRAMEWORK = {
    'TEST_REQUEST_DEFAULT_FORMAT': 'json',
    'DEFAULT_PAGINATION_CLASS':
        'terracommon.core.pagination.PagePagination',
    'PAGE_SIZE': 100,

    'DEFAULT_PARSER_CLASSES': (
        'rest_framework.parsers.JSONParser',
        'rest_framework.parsers.FormParser',
        'rest_framework.parsers.MultiPartParser'
    ),
    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework.renderers.JSONRenderer',
        'rest_framework.renderers.BrowsableAPIRenderer',
    ),
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.IsAuthenticated',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework_jwt.authentication.JSONWebTokenAuthentication',
        'rest_framework.authentication.SessionAuthentication',
        'rest_framework.authentication.BasicAuthentication',
    ),
}


# JWT configuration
# http://getblimp.github.io/django-rest-framework-jwt/
JWT_AUTH = {
    'JWT_ENCODE_HANDLER': 'rest_framework_jwt.utils.jwt_encode_handler',
    'JWT_DECODE_HANDLER': 'rest_framework_jwt.utils.jwt_decode_handler',
    'JWT_PAYLOAD_HANDLER': 'terracommon.accounts.jwt_payload.terra_payload_handler',
    'JWT_PAYLOAD_GET_USER_ID_HANDLER': 'rest_framework_jwt.utils.jwt_get_user_id_from_payload_handler',
    'JWT_RESPONSE_PAYLOAD_HANDLER': 'rest_framework_jwt.utils.jwt_response_payload_handler',
    # 'JWT_SECRET_KEY': settings.SECRET_KEY,
    'JWT_GET_USER_SECRET_KEY': None,
    'JWT_PUBLIC_KEY': None,
    'JWT_PRIVATE_KEY': None,
    'JWT_ALGORITHM': 'HS256',
    'JWT_VERIFY': True,
    'JWT_VERIFY_EXPIRATION': True,
    'JWT_LEEWAY': 0,
    'JWT_EXPIRATION_DELTA': timedelta(seconds=300),
    'JWT_AUDIENCE': None,
    'JWT_ISSUER': None,
    'JWT_ALLOW_REFRESH': True,
    'JWT_REFRESH_EXPIRATION_DELTA': timedelta(days=7),
    'JWT_AUTH_HEADER_PREFIX': 'JWT',
    'JWT_AUTH_COOKIE': None,
}

SERIALIZATION_MODULES = {
    'geojson': 'terracommon.terra.serializers.geojson',
}

WSGI_APPLICATION = PROJECT_NAME + '.wsgi.application'

# Internationalization
# https://docs.djangoproject.com/en/2.0/topics/i18n/
LOCALE_PATHS = (
    os.path.join(PROJECT_DIR, 'locales'),
)
LANGUAGE_CODE = 'fr-fr'
TIME_ZONE = 'Europe/Paris'
USE_I18N = True
USE_L10N = True
USE_TZ = True

FORMAT_MODULE_PATH = [
    PROJECT_NAME + '.formats',
]

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.0/howto/static-files/

STATIC_URL = '/static/'
STATICFILES_DIRS = ()
STATIC_ROOT = os.path.join(PUBLIC_DIR, 'static')

MEDIA_URL = '/media/'
MEDIA_ROOT = os.path.join(PUBLIC_DIR, 'media')

# Just to be easily override by children conf files.
LOGGING = copy.deepcopy(DEFAULT_LOGGING)

AUTH_USER_MODEL = 'accounts.TerraUser'

REQUEST_SCHEMA = {}

# Geometrical projection used internaly in database
# Note: Other projection than 4326 are not yet supported by some part of code
INTERNAL_GEOMETRY_SRID = 4326

MAX_TILE_ZOOM = 15 # Vector tiles are usable on display up to max zoom + 3
MIN_TILE_ZOOM = 10

STATES = Choices(
    ('DRAFT', 100, 'Draft'),
    ('SUBMITTED', 200, 'Submitted'),
    ('ACCEPTED', 300, 'Accepted'),
    ('REFUSED', -1, 'Refused'),
    ('CANCELLED', -100, 'Cancelled'),
    ('MISSING', 0, 'Missing'),
)
STATES.add_subset('MANUAL', (
    'DRAFT',
    'SUBMITTED',
    'ACCEPTED',
    'REFUSED',
    'CANCELLED',
))

TERRA_APPLIANCE_SETTINGS = {}
FRONT_URL = os.environ.get('FRONT_URL', '')
HOSTNAME = os.environ.get('HOSTNAME', '')

TERRA_TILES_HOSTNAMES = [
    HOSTNAME,
]

MEDIA_ACCEL_REDIRECT = False



# Generated by Django 2.0.9 on 2018-10-31 09:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('events', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='eventhandler',
            name='priority',
            field=models.PositiveIntegerField(default=10),
        ),
    ]



import csv
import json
import tempfile

from django.contrib.gis.geos import GEOSException, GEOSGeometry, Point
from django.test import TestCase
from rest_framework.test import APIClient

from terracommon.terra.models import Layer
from terracommon.terra.tests.factories import (FeatureFactory, LayerFactory,
                                               UserFactory)
from terracommon.terra.tests.utils import get_files_tests
from terracommon.terra.transformations import set_geometry_from_options


class LayerFromCSVDictReaderTestCase(TestCase):
    def setUp(self):
        self.layer = Layer.objects.create(name='fake layer')

    def get_csv_reader_from_dict(self, fieldnames, rows):
        self.tmp_file = tempfile.NamedTemporaryFile(mode='r+',
                                                    encoding='iso-8859-15',
                                                    delete=False)

        writer = csv.writer(self.tmp_file.file, )
        writer.writerow(fieldnames)
        writer.writerows(rows)
        self.tmp_file.file.seek(0)

        return csv.DictReader(self.tmp_file.file, )

    def test_missing_coordinates(self):
        reader = self.get_csv_reader_from_dict(
            ['CODGEO', 'Nb Pharmacies et parfumerie',
             'Dynamique Entrepreneuriale', 'Nb Résidences Principales',
             'Nb propriétaire', 'Nb Logement', 'Nb Résidences Secondaires',
             'Nb Log Vacants', 'Nb Occupants Résidence Principale',
             'Nb Femme', 'Nb Homme', 'Nb Mineurs', 'Nb Majeurs',
             'Nb Etudiants', 'CP'],
            [[1006, 0, 42, 41, 28, 57, 13, 3, 86, 86, 86, 101, 71, 2, 1], ]
        )

        initial = self.layer.features.all().count()
        self.layer.from_csv_dictreader(reader=reader,
                                       options=[],
                                       operations=[],
                                       pk_properties=['CODGEO'])

        expected = initial
        self.assertEqual(self.layer.features.all().count(), expected)

    def test_init_options(self):
        """Create fake features to test features reinit"""
        for i in range(2):
            self.layer.features.create(geom=Point(),
                                       properties={'SIREN': '', 'NIC': ''}, )
        self.assertEqual(self.layer.features.all().count(), 2)

        reader = self.get_csv_reader_from_dict(
            ['SIREN', 'NIC', 'L1_NORMALISEE', 'L2_NORMALISEE',
             'L3_NORMALISEE', 'x', 'y'],
            [['518521414', '00038', '11 RUE DU MARCHIX', '44000 NANTES',
              'France', '-1.560408', '47.218658'],
             ['518521414', '00053', '52 RUE JACQUES BABINET', '31100 TOULOUSE',
              'France', '1.408246', '43.575224'],
             ['813792686', '00012', 'BOIS DE TULLE', '32700 LECTOURE',
              'France', '1.538103', '45.17995'],
             ['822869632', '00023', '52 RUE JACQUES BABINET', '31100 TOULOUSE',
              'France', '1.408246', '43.575224']]
        )

        options = {
            'longitude': 'x',
            'latitude': 'y',
        }
        operations = [
            set_geometry_from_options,
        ]

        self.layer.from_csv_dictreader(reader=reader,
                                       options=options,
                                       operations=operations,
                                       pk_properties=['SIREN', 'NIC'],
                                       init=True)

        # Init mode only create new items, it does not reset database
        self.assertEqual(self.layer.features.all().count(), 6)

        feature = self.layer.features.get(properties__SIREN='813792686',
                                          properties__NIC='00012')
        self.assertEqual(feature.properties.get('L1_NORMALISEE', ''),
                         'BOIS DE TULLE')

    def test_create_update(self):
        self.layer.features.create(
            geom=Point(1.405812, 43.574511),
            properties={
                'SIREN': '437582422',
                'NIC': '00097',
                'L1_NORMALISEE': '36 RUE JACQUES BABINET',
                'L2_NORMALISEE': '31100 TOULOUSE',
                'L3_NORMALISEE': 'France',
            },
        )
        initial = self.layer.features.all().count()

        reader = self.get_csv_reader_from_dict(
            ['SIREN', 'NIC', 'L1_NORMALISEE', 'L2_NORMALISEE',
             'L3_NORMALISEE', 'long', 'lat'],
            [['437582422', '00097', '52 RUE JACQUES BABINET', '31100 TOULOUSE',
              'France', '1.408246', '43.575224'],
             ['518521414', '00038', '11 RUE DU MARCHIX', '44000 NANTES',
              'France', '-1.560408', '47.218658']]
        )

        options = {
            'longitude': 'long',
            'latitude': 'lat',
        }
        operations = [
            set_geometry_from_options,
        ]

        self.layer.from_csv_dictreader(reader=reader,
                                       options=options,
                                       operations=operations,
                                       pk_properties=['SIREN', 'NIC'])

        expected = initial + 1
        self.assertEqual(self.layer.features.all().count(), expected)

        feature = self.layer.features.get(properties__SIREN='437582422',
                                          properties__NIC='00097')
        self.assertEqual(feature.properties.get('L1_NORMALISEE', ''),
                         '52 RUE JACQUES BABINET')

    def test_operations(self):
        self.layer.features.create(
            geom=Point(1.405812, 43.574511),
            properties={
                'SIREN': '437582422',
                'NIC': '00097',
                'L1_NORMALISEE': '36 RUE JACQUES BABINET',
                'L2_NORMALISEE': '31100 TOULOUSE',
                'L3_NORMALISEE': 'France',
            },
        )

        reader = self.get_csv_reader_from_dict(
            ['SIREN', 'NIC', 'L1_NORMALISEE', 'L2_NORMALISEE',
             'L3_NORMALISEE', 'x', 'y'],
            [['437582422', '00097', '52 RUE JACQUES BABINET', '31100 TOULOUSE',
              'France', '1.408246', '43.575224']]
        )

        def custom_transformation(feature_args, options):
            properties = feature_args.get('properties')
            if properties.get('x'):
                properties['long'] = properties['x']
                del properties['x']
            if properties.get('y'):
                properties['lat'] = properties['y']
                del properties['y']
            feature_args['properties'] = properties

        options = {
            'latitude': 'lat',
            'longitude': 'long',
        }
        operations = [
            custom_transformation,
            set_geometry_from_options,
        ]
        self.layer.from_csv_dictreader(reader=reader,
                                       options=options,
                                       operations=operations,
                                       pk_properties=['SIREN', 'NIC'])

        feature = self.layer.features.get(properties__SIREN='437582422',
                                          properties__NIC='00097')
        self.assertEqual((1.408246, 43.575224), feature.geom.coords)


class LayerFromGeojsonTestCase(TestCase):
    def setUp(self):
        self.layer = LayerFactory()
        self.user = UserFactory()
        self.client.force_login(self.user)

    def test_import_geojson_with_projection(self):
        with_projection = """{"type": "FeatureCollection", "crs":
                             { "type": "name", "properties": { "name":
                             "urn:ogc:def:crs:OGC:1.3:CRS84" } },
                             "features": []}"""
        self.layer.from_geojson(with_projection, "01-01", "01-01")

    def test_import_geojson_withtout_projection(self):
        without_projection = """{"type": "FeatureCollection",
                                "features": []}"""
        self.layer.from_geojson(without_projection, "01-01", "01-01")

        with_bad_projection = """{"type": "FeatureCollection", "crs":
                                 { "type": "name", "properties": { "name":
                                 "BADPROJECTION" } }, "features": []}"""
        with self.assertRaises(GEOSException):
            self.layer.from_geojson(with_bad_projection, "01-01", "01-01")


class LayerFromShapefileTestCase(TestCase):
    def setUp(self):
        self.layer = LayerFactory()
        self.user = UserFactory()
        self.client.force_login(self.user)

    def test_shapefile_import(self):
        layer = LayerFactory()
        shapefile_path = get_files_tests('shapefile-WGS84.zip')

        with open(shapefile_path, 'rb') as shapefile:
            layer.from_shapefile(shapefile)

        self.assertEqual(8, layer.features.all().count())


class LayerExportGeometryTestCase(TestCase):
    def setUp(self):
        self.layer = LayerFactory()
        self.fake_geometry = {
            "type": "Point",
            "coordinates": [
                2.,
                45.
            ]
        }

    def test_to_geojson(self):
        # Create at least one feature in the layer, so it's not empty
        FeatureFactory(layer=self.layer)
        FeatureFactory(
            layer=self.layer,
            geom=GEOSGeometry(json.dumps(self.fake_geometry)),
            properties={'number': 1, 'digit': 34},
        )
        self.assertEqual(str(self.layer.to_geojson()['features'][0]['geometry']),
                         "{'type': 'Point', 'coordinates': [2.4609375, 45.583289756006316]}")
        self.assertEqual(str(self.layer.to_geojson()['features'][1]['geometry']),
                         "{'type': 'Point', 'coordinates': [2.0, 45.0]}")
        self.assertEqual(str(self.layer.to_geojson()['features'][1]['properties']),
                         "{'digit': 34, 'number': 1}")


class LayerSettingsTestCase(TestCase):
    def setUp(self):
        self.client = APIClient()
        self.layer = LayerFactory()
        self.user = UserFactory()
        self.client.force_authenticate(self.user)

    def test_layer_settings(self):
        with self.assertRaises(KeyError):
            self.layer.layer_settings('foo', 'bar', '666')

        with self.assertRaises(KeyError):
            self.layer.layer_settings('tiles', 'minzoom')

        self.assertEqual(
            self.layer.layer_settings_with_default('tiles', 'minzoom'),
            0
        )

    def test_set_layer_settings(self):
        with self.assertRaises(KeyError):
            self.layer.layer_settings('foo', 'bar')

        self.layer.set_layer_settings('foo', 'bar', 123)

        self.assertEqual(
            self.layer.layer_settings('foo', 'bar'),
            123
        )

        self.assertEqual(
            self.layer.layer_settings_with_default('foo', 'bar'),
            123
        )



# Generated by Django 2.0.9 on 2018-11-21 10:08

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import versatileimagefield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('terra', '0015_auto_20181005_1302'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Campaign',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('label', models.CharField(max_length=100, verbose_name='Label')),
                ('assignee', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='assigned_campaigns', to=settings.AUTH_USER_MODEL, verbose_name='Assigned to')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='campaigns', to=settings.AUTH_USER_MODEL, verbose_name='Owner')),
            ],
            options={
                'ordering': ['-created_at'],
                'permissions': (('manage_all_campaigns', 'Can manage all campaigns'),),
            },
        ),
        migrations.CreateModel(
            name='Document',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('properties', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict, verbose_name='Properties')),
                ('file', models.FileField(upload_to='', verbose_name='File')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='documents', to=settings.AUTH_USER_MODEL, verbose_name='Owner')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Picture',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('state', models.IntegerField(default=100, verbose_name='State')),
                ('properties', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict, verbose_name='Properties')),
                ('file', versatileimagefield.fields.VersatileImageField(upload_to='', verbose_name='File')),
                ('date', models.DateTimeField(verbose_name='Date')),
                ('remarks', models.TextField(max_length=350, verbose_name='Remarks')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='pictures', to=settings.AUTH_USER_MODEL, verbose_name='Owner')),
            ],
            options={
                'permissions': (('change_state_picture', 'Is able to change the picture state'),),
                'get_latest_by': 'date',
            },
        ),
        migrations.CreateModel(
            name='Theme',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('label', models.CharField(max_length=100, verbose_name='Label')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Viewpoint',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('label', models.CharField(max_length=100, verbose_name='Label')),
                ('properties', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict, verbose_name='Properties')),
                ('point', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='points', to='terra.Feature')),
                ('themes', models.ManyToManyField(blank=True, related_name='viewpoints', to='tropp.Theme')),
            ],
            options={
                'ordering': ['-created_at'],
                'permissions': (('can_download_pdf', 'Is able to download a pdf document'),),
            },
        ),
        migrations.AddField(
            model_name='picture',
            name='viewpoint',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pictures', to='tropp.Viewpoint'),
        ),
        migrations.AddField(
            model_name='document',
            name='viewpoint',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='documents', to='tropp.Viewpoint'),
        ),
        migrations.AddField(
            model_name='campaign',
            name='viewpoints',
            field=models.ManyToManyField(related_name='campaigns', to='tropp.Viewpoint'),
        ),
        migrations.AddIndex(
            model_name='picture',
            index=models.Index(fields=['viewpoint', 'date'], name='tropp_pictu_viewpoi_d2671c_idx'),
        ),
    ]



# Generated by Django 2.0.13 on 2019-07-15 09:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tropp', '0004_auto_20190711_1801'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='document',
            name='owner',
        ),
        migrations.RemoveField(
            model_name='document',
            name='viewpoint',
        ),
        migrations.DeleteModel(
            name='Document',
        ),
    ]



from rest_framework import permissions


class IsOwnerOrStaff(permissions.BasePermission):
    def has_object_permission(self, request, view, object):
        if request.user.is_staff or request.user.is_superuser:
            return True

        return object.owner == request.user



from rest_framework import routers

from .views import DocumentTemplateViewSets

app_name = 'document_generator'

router = routers.SimpleRouter()

router.register(r'document-template',
                DocumentTemplateViewSets,
                base_name='document')

urlpatterns = router.urls



# Generated by Django 2.0.9 on 2018-10-23 15:22

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0002_auto_20180911_0818'),
    ]

    operations = [
        migrations.AlterField(
            model_name='terrauser',
            name='uuid',
            field=models.CharField(default=uuid.uuid4, editable=False, max_length=255, unique=True, verbose_name='unique identifier'),
        ),
    ]



# -*- coding: utf-8 -*-
import logging
import os
from datetime import timedelta

from django.utils import six

from . import *

SECRET_KEY = 'dev-dev-dev-dev-dev-dev-dev'

ALLOWED_HOSTS = []
DEBUG = True

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

INSTALLED_APPS += (  # noqa
    'django_extensions',
)

INTERNAL_IPS = ('127.0.0.1',)  # Used by app debug_toolbar

# Add the Python core NullHandler to be available when needed
LOGGING['handlers']['null'] = {  # noqa
    'level': logging.NOTSET,
    'class': 'logging.NullHandler',
}

DATABASES = {
    'default': {
        'ENGINE': 'django.contrib.gis.db.backends.postgis',
        'NAME': os.environ.get('POSTGRES_DB'),
        'USER': os.environ.get('POSTGRES_USER'),
        'PASSWORD': os.environ.get('POSTGRES_PASSWORD'),
        'HOST': 'db',
        'PORT': '',
    }
}

# Force every loggers to use console handler only. Note that using 'root'
# logger is not enough if children don't propage.
for logger in six.itervalues(LOGGING['loggers']):  # noqa
    logger['handlers'] = ['console']
# Log every level.
LOGGING['handlers']['console']['level'] = logging.NOTSET  # noqa

CORS_ORIGIN_ALLOW_ALL = True

JWT_AUTH['JWT_EXPIRATION_DELTA'] = timedelta(hours=12)



import base64
import binascii
import logging

import magic
from django.db.models.fields.files import FieldFile
from rest_framework import serializers

logger = logging.getLogger(__name__)


class FileBase64Field(serializers.FileField):
    def to_representation(self, value):
        if not value:
            return None

        if not isinstance(value, FieldFile):
            raise TypeError(
                f'Expect a django FieldFile, instead get {type(value)}'
            )

        with open(value.path, mode='rb') as f:
            return (f'data:{magic.from_file(value.path, mime=True)};'
                    f'base64,{(base64.b64encode(f.read())).decode("utf-8")}')

    def to_internal_value(self, data):
        try:
            encoded = data.split(",", 1)[1]

        # Caught the error to log it then re-raised it
        except IndexError:
            logger.warning(
                f"cannot read document {data}"
            )
            raise serializers.ValidationError(
                'document field must be "data:<mime>;base64,<string>" format'
            )

        else:
            try:

                decoded = base64.b64decode(encoded)
            # Caught the error to log it then re-raised it
            except binascii.Error:
                logger.warning(f'{data} is not a base64 format')
                raise serializers.ValidationError(
                    f'expected a base64, get instead {type(data)}'
                )

        return decoded



# Generated by Django 2.0.8 on 2018-10-04 08:52

import django.contrib.postgres.fields.jsonb
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0009_alter_user_last_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='DataStore',
            fields=[
                ('key', models.CharField(max_length=255, primary_key=True, serialize=False)),
                ('value', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
            ],
        ),
        migrations.CreateModel(
            name='DataStorePermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('prefix', models.CharField(max_length=255)),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
                ('permission', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Permission')),
            ],
            options={
                'permissions': (('can_read_datastore', "Is able to read all datastore's elements"), ('can_readwrite_datastore', 'Is able to write in datastore')),
            },
        ),
    ]



import logging

from rest_framework import serializers

from .fields import FileBase64Field
from .models import DataStore, RelatedDocument

logger = logging.getLogger(__name__)


class DataStoreSerializer(serializers.ModelSerializer):
    class Meta:
        model = DataStore
        fields = '__all__'
        read_only_fields = ('key', )


class RelatedDocumentSerializer(serializers.ModelSerializer):
    document = FileBase64Field()

    class Meta:
        model = RelatedDocument
        fields = ('key', 'document', )


class RelatedDocumentPDFSerializer(RelatedDocumentSerializer):
    document = serializers.FileField(use_url=False)


class RelatedDocumentFileSerializer(RelatedDocumentSerializer):
    document = serializers.FileField()

    class Meta:
        model = RelatedDocument
        fields = ('key', 'document', 'properties')



from rest_framework.permissions import SAFE_METHODS, IsAuthenticated

from .models import DataStorePermission


class IsAuthenticatedAndDataStoreAllowed(IsAuthenticated):
    def has_object_permission(self, request, view, obj):
        permissions = ['can_readwrite_datastore', ]

        if request.method in SAFE_METHODS:
            permissions.append('can_read_datastore')

        perms = DataStorePermission.objects.filter(
            permission__codename__in=permissions,
            group__in=request.user.groups.all()
        )

        return any([obj.key.startswith(perm.prefix) for perm in perms])



# Generated by Django 2.0.13 on 2019-07-25 14:00

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('tropp', '0006_picture_order_by'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='picture',
            options={'get_latest_by': 'date', 'ordering': ['-date'], 'permissions': (('change_state_picture', 'Is able to change the picture state'),)},
        ),
        migrations.AlterField(
            model_name='picture',
            name='date',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date'),
            preserve_default=False,
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import terracommon


def read(fname):
    return open(fname).read()


setup(
    name="terra-common",
    version=terracommon.__version__,
    author=terracommon.__author__,
    author_email=terracommon.__email__,
    description="The Terra API",
    url="https://gitlab.makina-corpus.net/Terralego/Core/terra-back/",
    packages=find_packages(
        exclude=["terracommon.project", "terracommon.project.*", "*.tests"]
    ),
    include_package_data=True,
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development",
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=[
        "psycopg2>=2.7,<2.8",
        "Django>=2.1,<2.2",
        "djangorestframework>=3.8,<3.9",
        "djangorestframework-gis>=0.14,<0.15",
        "djangorestframework-jwt>=1.11,<1.12",
        "drf-yasg>=1.9,<1.10",
        "django-cors-headers>=2.2,<2.3",
        "simpleeval>=0.9,<1.0",
        "requests>=2.19,<2.20",
        "mercantile>=1.0,<1.1",
        "django-url-filter>=0.3,<0.4",
        "Fiona>=1.7,<1.8",
        "python-magic>=0.4,<0.5",
        "docxtpl>=0.5,<0.6",
        "Pillow>=5.3.0,<6.0.0",
        "django-storages>=1.7,<1.8",
        "django-versatileimagefield>=1.10,<2.0",
        "boto3>=1.9,<=1.10",
        "deepmerge<=1.0",
        "weasyprint>=44",
        "jsonschema>=3.0,<3.1",
        "docutils<0.15",
    ],
)



# Generated by Django 2.0.9 on 2018-11-20 10:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('terra', '0015_auto_20181005_1302'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='feature',
            options={'ordering': ['id']},
        ),
        migrations.AlterModelOptions(
            name='featurerelation',
            options={'ordering': ['id']},
        ),
        migrations.AlterModelOptions(
            name='layer',
            options={'ordering': ['id'], 'permissions': (('can_update_features_properties', 'Is able update geometries properties'),)},
        ),
        migrations.AlterModelOptions(
            name='layerrelation',
            options={'ordering': ['id']},
        ),
    ]



# Generated by Django 2.0.7 on 2018-08-24 10:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('trrequests', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='userrequest',
            options={'permissions': (('can_create_requests', 'Is able to create a new requests'), ('can_read_self_requests', 'Is able to get own requests'), ('can_read_all_requests', 'Is able to get all requests'), ('can_comment_requests', 'Is able to comment an user request'), ('can_internal_comment_requests', 'Is able to add comments not visible by users'), ('can_read_comment_requests', 'Is allowed to read only non-internal comments'), ('can_change_state_requests', 'Is authorized to change the request state'), ('can_download_all_pdf', 'Is able to download a pdf document'))},
        ),
    ]



from django.conf import settings
from django.http import HttpResponseNotFound
from django.urls import path
from drf_yasg import openapi
from drf_yasg.views import get_schema_view
from rest_framework import permissions, routers
from rest_framework_jwt import views as auth_views

from .tiles.views import MVTView, TilejsonView
from .views import (FeatureRelationViewSet, FeatureViewSet,
                    LayerRelationViewSet, LayerViewSet)

app_name = 'terra'

schema_view = get_schema_view(
   openapi.Info(
      title="Terra PI",
      default_version='v1',
      description="The future of Makina Corpus",
   ),
   public=True,
   permission_classes=(permissions.AllowAny, ),
)

urlpatterns = [
    # auth
    path('auth/obtain-token/',
         auth_views.obtain_jwt_token,
         name='token-obtain'),
    path('auth/verify-token/',
         auth_views.verify_jwt_token,
         name='token-verify'),
    path('auth/refresh-token/',
         auth_views.refresh_jwt_token,
         name='token-refresh'),
    path(r'layer/<str:group>/tilejson',
         TilejsonView.as_view(),
         name='group-tilejson'),
    path(r'layer/<str:group>/tiles/<int:z>/<int:x>/<int:y>/',
         MVTView.as_view(),
         name='group-tiles'),
    # Fake pattern to be able to reverse this
    path(r'layer/<str:group>/tiles/{z}/{x}/{y}/',
         lambda request, group: HttpResponseNotFound(),
         name='group-tiles-pattern'),
]

router = routers.SimpleRouter()

router.register(r'layer', LayerViewSet)
router.register(r'layer/(?P<layer>[\d\w\-_]+)/feature', FeatureViewSet,
                base_name='feature')
router.register(r'layer_relation', LayerRelationViewSet)
router.register(r'layer_relation/(?P<layerrelation_pk>\d+)/feature_relation',
                FeatureRelationViewSet)

urlpatterns += router.urls

if settings.DEBUG or settings.SWAGGER_ENABLED:
    urlpatterns += [
        # schemas
        path('swagger/',
             schema_view.with_ui('swagger', cache_timeout=0),
             name='schema-swagger-ui'),
        path('redoc/',
             schema_view.with_ui('redoc', cache_timeout=0),
             name='schema-redoc'),
    ]



from enum import IntEnum


class GeometryTypes(IntEnum):
    Point = 0
    LineString = 1
    # LinearRing 2
    Polygon = 3
    MultiPoint = 4
    MultiLineString = 5
    MultiPolygon = 6
    GeometryCollection = 7

    @classmethod
    def choices(cls):
        return [(geom_type.value, geom_type) for geom_type in cls]



# Generated by Django 2.0.13 on 2019-07-15 12:40

from django.db import migrations
import django.db.models.expressions


class Migration(migrations.Migration):

    dependencies = [
        ('tropp', '0005_remove_document'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='picture',
            options={'get_latest_by': 'date', 'ordering': [django.db.models.expressions.OrderBy(django.db.models.expressions.F('date'), descending=True, nulls_last=True)], 'permissions': (('change_state_picture', 'Is able to change the picture state'),)},
        ),
    ]



# Generated by Django 2.0.8 on 2018-08-17 08:01

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='EventHandler',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('action', models.CharField(max_length=255)),
                ('handler', models.CharField(max_length=255)),
                ('settings', django.contrib.postgres.fields.jsonb.JSONField(default={})),
            ],
        ),
    ]



import coreapi
import coreschema
from django.db.models.expressions import OrderBy, RawSQL
from django.utils.dateparse import parse_date
from rest_framework import filters
from rest_framework.exceptions import ValidationError
from rest_framework.filters import OrderingFilter
from url_filter.integrations.drf import DjangoFilterBackend


class JSONFieldOrderingFilter(OrderingFilter):

    def get_ordering(self, request, queryset, view):
        ordering = super().get_ordering(request, queryset, view)

        if not ordering:
            ordering = []

        params = request.query_params.get(self.ordering_param)
        if params:
            fields = [param.strip() for param in params.split(',')]
            for field in fields:
                json_nested = field.split('__')
                model_field = json_nested.pop(0)
                descending = False

                if (model_field in ordering or
                    not self.remove_invalid_fields(queryset,
                                                   [model_field, ],
                                                   view,
                                                   request)):
                    # The model_field must be an authorized field
                    continue

                if model_field.startswith('-'):
                    descending = True
                    model_field = model_field[1:]

                tpl = model_field + ''.join(
                    ['->>%s' for x in range(len(json_nested))])

                ordering.append(
                    OrderBy(RawSQL('lower({})'.format(tpl), json_nested),
                            descending=descending))

        if not ordering:
            ordering = None

        return ordering


class DateFilterBackend(filters.BaseFilterBackend):
    """
    Filter from date to date.

    You must specify which field to search on with `date_search_field` in
    your view.

    Example:
       date_search_field = 'created_at'
    """

    def get_schema_fields(self, view):
        super().get_schema_fields(view)
        search_field = getattr(view, 'date_search_field', None)
        return [
            coreapi.Field(
                name='date_from',
                required=False,
                location='query',
                schema=coreschema.String(
                    title="Begin date",
                    description=f"Begin date for {search_field}",
                    pattern='[0-9]{4}-[0-9]{2}-[0-9]{2}',
                ),
            ),
            coreapi.Field(
                name='date_to',
                required=False,
                location='query',
                schema=coreschema.String(
                    title="End date",
                    description=f"End date for {search_field}",
                    pattern='[0-9]{4}-[0-9]{2}-[0-9]{2}',
                ),
            ),
        ]

    def filter_queryset(self, request, queryset, view):
        search_field = getattr(view, 'date_search_field', None)
        date_from = self.parse_date(request.GET.get('date_from', None))
        date_to = self.parse_date(request.GET.get('date_to', None))

        if date_from and date_to and date_from > date_to:
            raise ValidationError

        if date_from is not None and date_to is not None:
            queryset = queryset.filter(
                **{f'{search_field}__range': (date_from, date_to)}
            )
        elif date_from is not None:
            queryset = queryset.filter(**{f'{search_field}__gte': date_from})
        elif date_to is not None:
            queryset = queryset.filter(**{f'{search_field}__lte': date_to})

        return queryset

    @staticmethod
    def parse_date(value):
        """
        Shamelessly taken from DateField.to_python

        :param value:
        :return:
        """
        if value is None:
            return value

        try:
            parsed = parse_date(value)
            if parsed is not None:
                return parsed
        except ValueError:
            raise ValidationError

        raise ValidationError


class SchemaAwareDjangoFilterBackend(DjangoFilterBackend):
    def get_schema_fields(self, view):
        """
        Get coreapi filter definitions

        Returns all schemas defined in filter_fields_schema attribute.
        """
        super().get_schema_fields(view)
        return getattr(view, 'filter_fields_schema', [])



from django.contrib.auth.tokens import default_token_generator
from django.contrib.auth.views import PasswordResetConfirmView
from rest_framework.permissions import (SAFE_METHODS, BasePermission,
                                        IsAuthenticated)


class TokenBasedPermission(BasePermission):
    get_user = PasswordResetConfirmView.get_user

    def has_permission(self, request, view):
        if 'uidb64' in request.GET and 'token' in request.GET:
            user = self.get_user(request.GET['uidb64'])

            if default_token_generator.check_token(user, request.GET['token']):
                view.request.user = user
                return True

        return False


class IsAuthenticatedPost(IsAuthenticated):
    def has_object_permission(self, request, view, *args, **kwargs):
        return self.has_permission(request, view)

    def has_permission(self, request, view):
        if request.method not in SAFE_METHODS:
            return super().has_permission(request, view)
        return False


class IsPostOrToken(TokenBasedPermission, IsAuthenticatedPost):
    def has_permission(self, *args):
        return (TokenBasedPermission.has_permission(self, *args)
                or IsAuthenticatedPost.has_permission(self, *args))



# Generated by Django 2.0.9 on 2018-11-23 09:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tropp', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='viewpoint',
            name='themes',
        ),
        migrations.DeleteModel(
            name='Theme',
        ),
    ]



from django.contrib.auth.models import Permission
from django.contrib.gis import admin

from . import models


@admin.register(models.Layer)
class LayerAdmin(admin.ModelAdmin):
    list_display = ('pk', 'name', 'group', 'geom_type')
    list_filter = ('group', 'geom_type')


@admin.register(models.Feature)
class FeatureAdmin(admin.OSMGeoAdmin):
    list_display = ('pk', 'identifier', 'layer', 'source', 'target')
    list_filter = ('layer', )


admin.site.register(models.FeatureRelation)
admin.site.register(Permission)



from django.conf import settings
from django.db.models import Q
from django.http.response import Http404, HttpResponseServerError
from django.shortcuts import get_object_or_404
from rest_framework import mixins, permissions, status, viewsets
from rest_framework.decorators import detail_route, list_route
from rest_framework.exceptions import PermissionDenied
from rest_framework.filters import SearchFilter
from rest_framework.response import Response
from url_filter.integrations.drf import DjangoFilterBackend

from terracommon.accounts.permissions import TokenBasedPermission
from terracommon.core.filters import JSONFieldOrderingFilter
from terracommon.events.signals import event
from terracommon.terra.helpers import get_media_response

from .models import UserRequest
from .serializers import CommentSerializer, UserRequestSerializer


class RequestViewSet(viewsets.ModelViewSet):
    serializer_class = UserRequestSerializer
    permission_classes = [permissions.IsAuthenticated, ]
    filter_backends = (SearchFilter, JSONFieldOrderingFilter,
                       DjangoFilterBackend,)
    search_fields = ('id', 'properties', 'expiry')
    filter_fields = ('state', 'reviewers', 'expiry')

    def get_queryset(self):
        if self.request.user.has_perm('trrequests.can_read_all_requests'):

            # Return  all non-draft request
            # except for the one the user is the owner.
            return UserRequest.objects.exclude(
                ~Q(owner=self.request.user)
                & Q(state=settings.STATES.DRAFT)
            )
        elif self.request.user.has_perm('trrequests.can_read_self_requests'):
            return UserRequest.objects.filter(
                Q(owner=self.request.user)
                | Q(reviewers__in=[self.request.user, ])
            ).distinct()
        return UserRequest.objects.none()

    def create(self, request, *args, **kwargs):
        if not self.request.user.has_perm('trrequests.can_create_requests'):
            raise PermissionDenied

        return super().create(request, *args, **kwargs)

    def perform_create(self, serializer):
        instance = serializer.save(owner=self.request.user)
        event.send(
            self.__class__,
            action="USERREQUEST_CREATED",
            user=self.request.user,
            instance=instance)

    def patch(self, request, *args, **kwargs):
        return super().partial_update(request, *args, **kwargs)

    @list_route(methods=['get'])
    def schema(self, request):
        if isinstance(settings.REQUEST_SCHEMA, dict):
            return Response(settings.REQUEST_SCHEMA)
        else:
            return HttpResponseServerError()

    @detail_route(methods=['get'])
    def read(self, request, pk):
        self.get_object().user_read(request.user)
        userrequest = UserRequestSerializer(
            self.get_object(),
            context={'request': self.request})

        return Response(
            userrequest.data,
            status=status.HTTP_200_OK)


class CommentViewSet(mixins.CreateModelMixin,
                     mixins.RetrieveModelMixin,
                     mixins.ListModelMixin,
                     viewsets.GenericViewSet):
    serializer_class = CommentSerializer
    permission_classes = [permissions.IsAuthenticated, ]

    def get_queryset(self, *args, **kwargs):
        request_pk = self.kwargs.get('request_pk')

        return get_object_or_404(
            UserRequest,
            pk=request_pk).get_comments_for_user(self.request.user)

    def perform_create(self, serializer):

        if 'is_internal' in serializer.validated_data:
            if (serializer.validated_data['is_internal'] is True
                    and not self.request.user.has_perm(
                        'trrequests.can_internal_comment_requests')):
                raise PermissionDenied(
                    'Permission missing to create internal comment')

            elif (serializer.validated_data['is_internal'] is False
                  and not self.request.user.has_perm(
                        'trrequests.can_comment_requests')):
                raise PermissionDenied(
                    'Permission missing to create public comment')

        auto_data = {
            'owner': self.request.user,
            'userrequest': get_object_or_404(UserRequest,
                                             pk=self.kwargs.get('request_pk'))
        }

        return serializer.save(**auto_data)

    @detail_route(methods=['get'], permission_classes=(TokenBasedPermission,))
    def attachment(self, request, request_pk=None, pk=None):
        comment = self.get_object()
        if not comment.attachment:
            raise Http404('Attachment does not exist')

        response = get_media_response(
            request, comment.attachment,
            headers={
                'Content-Disposition': (
                    'attachment;'
                    f' filename={comment.filename}'),
            }
        )

        return response



from io import StringIO

from django.core.management import call_command
from django.core.management.base import CommandError
from django.test import TestCase

from terracommon.terra.models import Layer
from terracommon.terra.tests.factories import LayerFactory
from terracommon.terra.tests.utils import get_files_tests


class ImportShapefileTest(TestCase):
    def test_default_group_nogroup(self):
        call_command(
            'import_shapefile',
            get_files_tests('shapefile-WGS84.zip'),
            '-i', 'ID_PG',
            verbosity=0)

        # Retrieve the layer
        layer = Layer.objects.all()[0]
        self.assertEqual('__nogroup__', layer.group)

        # Assert the identifier is not an UUID4
        self.assertTrue(len(str(layer.features.first().identifier)) < 32)

    def test_default_group_nogroup_rollback(self):
        # Sample ShapeFile
        shapefile_path = get_files_tests('shapefile-WGS84.zip')
        sample_shapefile = open(shapefile_path, 'rb')

        output = StringIO()
        call_command(
            'import_shapefile',
            f'-iID_PG',
            f'--dry-run',
            f'{sample_shapefile.name}',
            verbosity=1, stdout=output)
        self.assertIn("The created layer pk is", output.getvalue())
        # Retrieve the layer
        layer = Layer.objects.all()
        self.assertEqual(len(layer), 0)

    def test_reprojection(self):
        output = StringIO()
        call_command(
            'import_shapefile',
            get_files_tests('shapefile-RFG93.zip'),
            verbosity=1, stdout=output)

        # Retrieve the layer
        layer = Layer.objects.all()[0]
        self.assertEqual('__nogroup__', layer.group)

        # assert data was reprojected
        bbox = layer.features.first().get_bounding_box()
        self.assertTrue(-180 <= bbox[0])
        self.assertTrue(-90 <= bbox[1])
        self.assertTrue(bbox[2] <= 180)
        self.assertTrue(bbox[3] <= 90)

    def test_default_group(self):
        # Fake json
        foo_bar_json = get_files_tests('foo_bar.json')

        # Import a shapefile
        call_command(
            'import_shapefile',
            get_files_tests('shapefile-WGS84.zip'),
            '-i', 'ID_PG',
            verbosity=0)

        # Ensure old settings
        layer = Layer.objects.all()[0]
        self.assertNotEqual('new_name', layer.name)
        self.assertNotEqual('new_group', layer.group)
        self.assertNotEqual({'foo': 'bar'}, layer.schema)
        self.assertNotEqual({'foo': 'bar'}, layer.settings)

        # Change settings
        call_command(
            'layer_edit',
            '-pk', layer.pk,
            '-l', 'new_name',
            '-gr', 'new_group',
            '-ls', foo_bar_json
        )

        # Ensure new settings
        layer = Layer.objects.all()[0]
        self.assertEqual('new_name', layer.name)
        self.assertEqual('new_group', layer.group)
        self.assertEqual({'foo': 'bar'}, layer.settings)

    def test_schema_generated(self):
        call_command(
            'import_shapefile',
            get_files_tests('shapefile-WGS84.zip'),
            '-gs',
            verbosity=0)

        # Retrieve the layer
        layer = Layer.objects.get()

        # Assert schema properties are presents
        self.assertNotEqual(
            layer.schema.get('properties').keys() -
            ['ALTITUDE', 'ETIQUETTE', 'HAUTEUR', 'ID', 'ID_PG', 'NATURE', 'NOM',
             'ORIGIN_BAT', 'PUB_XDECAL', 'PUB_YDECAL', 'ROTATION', 'ROTATION_S',
             'XDECAL', 'XDECAL_SYM', 'YDECAL', 'YDECAL_SYM', 'Z_MAX', 'Z_MIN', ], True)

    def test_import_shapefile_layer_with_bad_settings(self):
        # Sample ShapeFile
        shapefile_path = get_files_tests('shapefile-WGS84.zip')
        sample_shapefile = open(shapefile_path, 'rb')
        bad_settings_json = get_files_tests('bad.json')
        # Change settings
        with self.assertRaises(CommandError) as error:
            call_command(
                'import_shapefile',
                '-iID_PG',
                '-gs', sample_shapefile.name,
                '-ls', bad_settings_json,
                verbosity=0
            )
        self.assertEqual("Please provide a valid layer settings file", str(error.exception))

    def test_import_shapefile_layer_with_pk_layer(self):
        # Sample ShapeFile
        layer = LayerFactory()
        self.assertEqual(len(layer.features.all()), 0)
        shapefile_path = get_files_tests('shapefile-WGS84.zip')
        sample_shapefile = open(shapefile_path, 'rb')
        call_command(
            'import_shapefile',
            f'--layer-pk={layer.pk}',
            '-iID_PG',
            '-gs', sample_shapefile.name,
            verbosity=0
        )
        self.assertEqual(len(layer.features.all()), 8)

    def test_import_shapefile_layer_with_wrong_pk_layer(self):
        # Sample ShapeFile
        shapefile_path = get_files_tests('shapefile-WGS84.zip')
        sample_shapefile = open(shapefile_path, 'rb')
        with self.assertRaises(CommandError) as error:
            call_command(
                'import_shapefile',
                f'--layer-pk=999',
                '-iID_PG',
                '-gs', sample_shapefile.name,
                verbosity=0
            )
        self.assertIn("Layer with pk 999 doesn't exist", str(error.exception))



from django.core.management import BaseCommand
from django.utils.translation import ugettext as _

from terracommon.events.signals import event


class Command(BaseCommand):
    help = _('Sends signal related to an action passed as parameter')

    def add_arguments(self, parser):
        parser.add_argument('-a', '--action',
                            required=False,
                            action='store',
                            dest='action',
                            help=_('Specify action name'),
                            )
        parser.add_argument('-i', '--instance',
                            required=False,
                            action='store',
                            dest='instance',
                            help=_('Specify instance pk'),
                            )
        parser.add_argument('-u', '--user',
                            required=False,
                            action='store',
                            dest='user',
                            help=_('Specify user pk'),
                            )
        parser.add_argument('-k', '--kwargs',
                            required=False,
                            default=[],
                            action='append',
                            dest='kwargs',
                            help=_('Allow to pass any type of argument'),
                            )

    def handle(self, *args, **options):
        overloaded_keys = ['action', 'instance', 'user']
        required_keys = ['action']

        kwargs = dict()

        for kwarg in options.get('kwargs'):
            key, value = kwarg.split(':')
            kwargs[key] = value

        for key in overloaded_keys:
            if options.get(key):  # Passed in argument
                kwargs[key] = options[key]
            elif not kwargs.get(key):  # Missing value
                if key in required_keys:
                    continue  # Keep the exception happen
                kwargs[key] = None

        event.send(self.__class__, **kwargs)



from rest_framework import routers

from .views import DataStoreViewSet

app_name = 'datastore'

router = routers.SimpleRouter()

router.register(r'datastore', DataStoreViewSet, base_name='datastore')

urlpatterns = router.urls



import json

from django.contrib.gis.gdal.error import GDALException
from django.contrib.gis.geos import (GEOSException, GEOSGeometry, LineString,
                                     Point)
from django.core.serializers import serialize
from django.db import transaction
from django.db.models import Q
from django.http import HttpResponse, HttpResponseBadRequest, JsonResponse
from django.utils.datastructures import MultiValueDictKeyError
from rest_framework import status, viewsets
from rest_framework.decorators import detail_route
from rest_framework.generics import get_object_or_404
from rest_framework.permissions import SAFE_METHODS
from rest_framework.response import Response

from .filters import JSONFieldFilterBackend
from .mixins import MultipleFieldLookupMixin
from .models import FeatureRelation, Layer, LayerRelation
from .routing.helpers import Routing
from .serializers import (FeatureRelationSerializer, FeatureSerializer,
                          LayerRelationSerializer, LayerSerializer)


class LayerViewSet(MultipleFieldLookupMixin, viewsets.ModelViewSet):
    queryset = Layer.objects.all()
    serializer_class = LayerSerializer
    lookup_fields = ('pk', 'name')

    @detail_route(methods=['get', 'post'],
                  url_path='shapefile')
    def shapefile(self, request, pk=None):
        layer = self.get_object()

        if request.method not in SAFE_METHODS:
            if request.user.has_perm('terra.can_import_layers'):
                try:
                    shapefile = request.data['shapefile']
                    with transaction.atomic():
                        layer.features.all().delete()
                        layer.from_shapefile(shapefile)
                        response = Response(status=status.HTTP_200_OK)
                except (ValueError, MultiValueDictKeyError):
                    response = Response(status=status.HTTP_400_BAD_REQUEST)
            else:
                self.permission_denied(request, 'Operation not allowed')
        else:
            if request.user.has_perm('terra.can_export_layers'):
                shape_file = layer.to_shapefile()

                if shape_file:
                    response = HttpResponse(content_type='application/zip')
                    response['Content-Disposition'] = (
                        'attachment; '
                        f'filename="{layer.name}.zip"')

                    response.write(shape_file.getvalue())
                else:
                    response = Response(status=status.HTTP_204_NO_CONTENT)
            else:
                self.permission_denied(request, 'Operation not allowed')
        return response

    @detail_route(methods=['get'], url_path='geojson')
    def to_geojson(self, request, pk=None):
        if request.user.has_perm('terra.can_export_layers'):
            layer = self.get_object()
            return JsonResponse(layer.to_geojson())
        else:
            self.permission_denied(request, 'Operation not allowed')

    @detail_route(methods=['post'])
    def route(self, request, pk=None):
        layer = self.get_object()
        callbackid = self.request.data.get('callbackid', None)

        try:
            geometry = GEOSGeometry(request.data.get('geom', None))
            if not isinstance(geometry, LineString):
                raise ValueError
            points = [Point(c, srid=geometry.srid) for c in geometry.coords]
        except (GEOSException, TypeError, ValueError):
            return HttpResponseBadRequest(
                content='Provided geometry is not valid LineString')

        routing = Routing(points, layer)
        route = routing.get_route()

        if not route:
            return Response(status=status.HTTP_204_NO_CONTENT)

        response_data = {
            'request': {
                'callbackid': callbackid,
                'geom': geometry.json,
            },
            'geom': route,
        }

        return Response(response_data, content_type='application/json')

    @detail_route(methods=['post'])
    def intersects(self, request, *args, **kwargs):
        layer = self.get_object()
        callbackid = self.request.data.get('callbackid', None)

        try:
            geometry = GEOSGeometry(request.data.get('geom', None))
        except (GEOSException, GDALException, TypeError, ValueError):
            return HttpResponseBadRequest(
                content='Provided geometry is not valid')

        response = {
            'request': {
                'callbackid': callbackid,
                'geom': geometry.json,
            },
            'results': json.loads(serialize('geojson',
                                  layer.features.intersects(geometry),
                                  fields=('properties',),
                                  geometry_field='geom',
                                  properties_field='properties')),
        }

        return Response(response)

    def partial_update(self, request, *args, **kwargs):
        if not request.user.has_perm('terra.can_update_features_properties'):
            self.permission_denied(request, 'Operation not allowed')

        layer = self.get_object()

        if 'features' in request.data:
            try:
                features = layer.update_geometries(request.data['features'])
                return Response(
                            json.loads(serialize('geojson',
                                       features,
                                       fields=('properties',),
                                       geometry_field='geom',
                                       properties_field='properties'))
                        )
            except (ValueError, KeyError):
                return HttpResponseBadRequest('An error occured parsing '
                                              'GeoJSON, verify your data')
        else:
            return HttpResponseBadRequest('Features are missing in GeoJSON')


class FeatureViewSet(viewsets.ModelViewSet):
    serializer_class = FeatureSerializer
    filter_backends = (JSONFieldFilterBackend, )
    filter_fields = ('properties', )
    lookup_field = 'identifier'

    def _get_layer(self):
        queryfilter = Q(name=self.kwargs.get('layer'))
        if self.kwargs.get('layer').isdigit():
            queryfilter |= Q(pk=self.kwargs.get('layer'))

        return get_object_or_404(Layer, queryfilter)

    def get_serializer_context(self):
        """
        Layer access in serializer (pk to insure schema generation)
        """
        context = super().get_serializer_context()
        layer = self._get_layer()
        context.update({'layer_pk': layer.pk})
        return context

    def get_queryset(self):
        layer = self._get_layer()
        return layer.features.all()

    def perform_create(self, serializer):
        layer = self._get_layer()
        serializer.save(layer_id=layer.pk)


class LayerRelationViewSet(viewsets.ModelViewSet):
    queryset = LayerRelation.objects.all()
    serializer_class = LayerRelationSerializer


class FeatureRelationViewSet(viewsets.ModelViewSet):
    queryset = FeatureRelation.objects.all()
    serializer_class = FeatureRelationSerializer



import os
from datetime import date

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.http.response import Http404
from django.shortcuts import get_object_or_404
from jinja2 import TemplateSyntaxError
from requests.exceptions import ConnectionError, HTTPError
from rest_framework import status, viewsets
from rest_framework.decorators import detail_route
from rest_framework.exceptions import PermissionDenied
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from terracommon.accounts.permissions import TokenBasedPermission
from terracommon.terra.helpers import get_media_response
from terracommon.trrequests.models import UserRequest

from .helpers import DocumentGenerator
from .models import DocumentTemplate, DownloadableDocument
from .serializers import DocumentTemplateSerializer


class DocumentTemplateViewSets(viewsets.ModelViewSet):
    queryset = DocumentTemplate.objects.none()
    serializer_class = DocumentTemplateSerializer
    permission_classes = (IsAuthenticated, )

    def get_queryset(self, *args, **kwargs):
        if self.request.user.has_module_perms('document_generator'):
            return DocumentTemplate.objects.all()
        return DocumentTemplate.objects.none()

    def create(self, request, *args, **kwargs):
        if not request.user.has_perm(
                'document_generator.can_upload_template'):
            raise PermissionDenied

        return super().create(request, *args, **kwargs)

    def update(self, request, *args, **kwargs):
        if not request.user.has_perm(
                'document_generator.can_update_template'):
            raise PermissionDenied

        return super().update(request, *args, **kwargs)

    def destroy(self, request, *args, **kwargs):
        if not request.user.has_perm(
                'document_generator.can_delete_template'):
            raise PermissionDenied

        return super().destroy(request, *args, **kwargs)

    @detail_route(methods=['get'], permission_classes=(TokenBasedPermission,))
    def file(self, request, pk=None):
        document = self.get_object().documenttemplate
        if not document:
            raise Http404('Attachment does not exist')

        response = get_media_response(
            request, document,
            headers={
                'Content-Disposition': (
                    'attachment;'
                    f' filename={document.name}'),
            }
        )

        return response

    @detail_route(methods=['get'],
                  url_name='pdf',
                  url_path='pdf/(?P<request_pk>[^/.]+)',
                  permission_classes=(TokenBasedPermission,))
    def pdf_creator(self, request, pk=None, request_pk=None):
        """ Insert data from user request into a template & convert it to pdf

        <pk>: template's id
        <request_pk>: user request's id
        """

        userrequest = get_object_or_404(UserRequest, pk=request_pk)
        template = get_object_or_404(DocumentTemplate, pk=pk)

        userrequest_type = ContentType.objects.get_for_model(
                                                        userrequest)
        downloadable_properties = {
            'document': template,
            'content_type': userrequest_type,
            'object_id': userrequest.pk,
        }
        if not ((request.user.is_superuser
                 or request.user.has_perm('trrequests.can_download_all_pdf')
                or request.user.downloadabledocument_set.filter(
                    **downloadable_properties).exists())):
            return Response(status=status.HTTP_404_NOT_FOUND)

        try:
            pdf_generator = DocumentGenerator(
                DownloadableDocument.objects.get(**downloadable_properties)
            )
            pdf_path = pdf_generator.get_pdf()
        except FileNotFoundError:
            return Response(status=status.HTTP_404_NOT_FOUND)
        except (ConnectionError, HTTPError):
            return Response(status=status.HTTP_503_SERVICE_UNAVAILABLE)
        except TemplateSyntaxError:
            return Response(
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
                data='malformed template'
            )
        else:
            pdf_url = os.path.join(settings.MEDIA_URL, pdf_path)

            filename = f'document_{date.today().__str__()}.pdf'
            response = get_media_response(
                request,
                {'path': pdf_path, 'url': pdf_url},
                headers={
                    'Content-Type': 'application/pdf',
                    'Content-disposition': f'attachment;filename={filename}'
                }
            )
            return response



from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.contrib.auth.tokens import default_token_generator
from django.db import transaction
from django.db.utils import IntegrityError
from django.shortcuts import get_object_or_404
from rest_framework import permissions, status
from rest_framework.decorators import detail_route
from rest_framework.filters import SearchFilter
from rest_framework.generics import RetrieveUpdateAPIView
from rest_framework.parsers import JSONParser
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.viewsets import ModelViewSet
from url_filter.integrations.drf import DjangoFilterBackend

from terracommon.core.filters import JSONFieldOrderingFilter
from terracommon.events.signals import event

from .forms import PasswordSetAndResetForm
from .serializers import (PasswordChangeSerializer, PasswordResetSerializer,
                          TerraUserSerializer, UserProfileSerializer)

UserModel = get_user_model()


class UserProfileView(RetrieveUpdateAPIView):
    serializer_class = UserProfileSerializer
    permission_classes = (IsAuthenticated,)

    def get_object(self):
        return self.request.user

    def get_queryset(self):
        return get_user_model().objects.none()


class UserRegisterView(APIView):
    permission_classes = (AllowAny, )

    def post(self, request):
        try:
            form = PasswordSetAndResetForm(data=request.data)
            if form.is_valid():
                opts = {
                    'token_generator': default_token_generator,
                    'from_email': settings.DEFAULT_FROM_EMAIL,
                    'email_template_name':
                        'registration/registration_email.txt',
                    'subject_template_name':
                        'registration/registration_email_subject.txt',
                    'request': self.request,
                    'html_email_template_name':
                        'registration/registration_email.html',
                }

                with transaction.atomic():
                    user = get_user_model().objects.create(
                        **{
                            get_user_model().EMAIL_FIELD: (
                                request.data['email']
                            ),
                            'is_active': self._user_default_status,
                        })
                    user.set_unusable_password()
                    user.save()

                form.save(**opts)

                serializer = TerraUserSerializer(user)
                event.send(
                    self.__class__,
                    action="USER_CREATED",
                    user=user,
                    instance=user
                )
                return Response(serializer.data, status=status.HTTP_200_OK)
            else:
                return Response(data=form.errors,
                                status=status.HTTP_400_BAD_REQUEST)
        except IntegrityError:
            # If user already exists, email to reset the password instead
            form.save(**opts)
            return Response({}, status=status.HTTP_200_OK)

    @property
    def _user_default_status(self):
        try:
            return settings.TERRA_USER_CREATION_STATUS
        except AttributeError:
            return False


class UserSetPasswordView(APIView):
    permission_classes = (AllowAny, )

    def post(self, request, uidb64, token):
        serializer = PasswordResetSerializer(uidb64, token, data=request.data)
        serializer.is_valid(raise_exception=True)

        serializer.save()

        user_serializer = UserProfileSerializer()
        return Response(user_serializer.to_representation(serializer.user))


class UserChangePasswordView(APIView):
    permission_classes = (IsAuthenticated, )

    def post(self, request):
        serializer = PasswordChangeSerializer(request.user, data=request.data)
        serializer.is_valid(raise_exception=True)

        serializer.save()

        user_serializer = UserProfileSerializer()
        return Response(user_serializer.to_representation(serializer.user))


class SettingsView(APIView):
    permission_classes = ()
    authentication_classes = ()

    def get(self, request):
        terra_settings = {
            'states': {
                y: x
                for x, y in settings.STATES.VALUE_TO_CONST.items()
                },
            'jwt_delta': settings.JWT_AUTH['JWT_EXPIRATION_DELTA']
        }

        terra_settings.update(settings.TERRA_APPLIANCE_SETTINGS)

        return Response(terra_settings)


class UserInformationsView(APIView):
    def get(self, request):
        user = self.request.user
        return Response(TerraUserSerializer(user).data)


class UserViewSet(ModelViewSet):
    permission_classes = (permissions.IsAuthenticated, )
    parser_classes = (JSONParser, )
    serializer_class = TerraUserSerializer
    queryset = UserModel.objects.none()
    filter_backends = (DjangoFilterBackend, JSONFieldOrderingFilter,
                       SearchFilter, )
    search_fields = ('uuid', 'email', 'properties', )
    filter_fields = ('uuid', 'email', 'properties', 'groups', 'is_superuser',
                     'is_active', 'is_staff', 'date_joined')

    def get_queryset(self):
        if self.request.user.has_perm('accounts.can_manage_users'):
            return UserModel.objects.all()

        return self.queryset

    @detail_route(methods=['post', ])
    def groups(self, request, pk=None):
        user = get_object_or_404(UserModel, pk=pk)

        defined_groups = []

        for group in request.data['groups']:
            defined_groups.append(get_object_or_404(Group, name=group))

        user.groups.set(defined_groups)

        return Response(
            [g.name for g in defined_groups],
            status=status.HTTP_200_OK)
